/*
 	This file is part of DataFox, a bandwidth usage monitor for
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2011 Brian Fernandes <infernalproteus@gmail.com>

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/
datafox_constants.keyExp = new RegExp('.*key=([0-9]+).*', 'gi');
datafox_constants.dataExp = new RegExp("<tr>([\\s\\S]+?)</tr>", "i");
datafox_constants.numExp = new RegExp(">[0-9]*\\.?[0-9]+", "gi");

//constants for the data.bsnl.in site
datafox_constants.logFrmExp = new RegExp('<form.*action="(.*)".*name="LoginForm"', 'gi');
datafox_constants.servicesExp = new RegExp('<a.*href="([^"]*)".*View Your PostPaid', "gi");
datafox_constants.searchFrmExp = new RegExp('<form.*action="(.*)".*id="usageDetailsSearchForm"', 'gi');
//datafox_constants.bbIdExp = new RegExp('<option[^\\d]*(\\d{3,})', 'gi');
datafox_constants.bbIdExp = new RegExp('id\\s*=\\s*"serviceUserId".*?<option[^"]*"([^"]*)', 'gi');

DFox.prototype.bsnlLogin = function() {
	if (this.domain) {
		if (this.domain == "bbservice.bsnl.in") {
			//migrate older settings to new portal
			this.domain = "data.bsnl.in";
		}
		if (this.domain == "data.bsnl.in") {
			this.bsnlPortalType = 2;
			this.phase = this.FIRST_CONTACT;
			this.httpUtils.talkToServer("http://" + this.domain + "/wps/portal", null, null, null, this);
		} else if (this.isSelfcare(this.domain)) {
			this.bsnlPortalType = 3;
			this.phase = this.FIRST_CONTACT;
			this.bsnlResponseHandler("");
		} else {
			//make first contact
			this.phase = this.FIRST_CONTACT;
			this.bsnlPortalType = 0;
			this.httpUtils.talkToServer("http://" + this.domain, null, null, null, this);
		}
	} else {
		this.bsnlPortalType = 0;
		this.phase = this.ZERO_CONTACT;
		this.httpUtils.talkToServer("http://sancharnet.in/cgi-bin/redirect.pl", null, null, null, this, true);
	}
};

DFox.prototype.isSelfcare = function(domain) {
	return (domain == "selfcare.ndc.bsnl.co.in" || domain == "selfcare.wdc.bsnl.co.in");
};

 DFox.prototype.redirectIP = function(ip) {
	this.log("Redirected to " + ip);
	this.domain = ip;
	this.phase = this.FIRST_CONTACT;
	this.responseHandler(null, null);
};

DFox.prototype.bsnlResponseHandler = function(data) {
	if (this.bsnlPortalType == 0) {
		this.bsnlResponseHandler_sancharnet(data);
	} else if (this.bsnlPortalType == 2) {
		this.bsnlResponseHandler_databsnlin(data);
	} else if (this.bsnlPortalType == 3) {
		this.bsnlResponseHandler_ndc(data);
	}
};

DFox.prototype.bsnlResponseHandler_ndc = function(data) {
	switch (this.phase) {
		case this.FIRST_CONTACT:			  
			var pData = "login_submit=on&login_do_redirect=1&no_cert_storing=on&uidPasswordLogon=Log+on&j_authscheme=default&j_user=" + this.username + "&j_password=" + this.password;
			this.phase = this.LOGIN;
			this.openURL = "http://" + this.domain + "/irj/portal";
			this.httpUtils2.talkToServer(this.openURL, pData, null, null, this);
			break;
		case this.LOGIN:
			if (/User.*authentication.*failed/.test(data)) {
				this.setState(this.ERROR);
				break;				
			}
			this.phase = this.SET_PARAM;
			this.wid = "WID" + (new Date()).getTime();
			this.httpUtils2.talkToServer("http://" + this.domain + "/irj/servlet/prt/portal/prtroot/pcd!3aportal_content!2fcom.bsnl.BSNL!2fcom.bsnl.BSNL2!2fcom.bsnl.CustomDesktop!2fWSC_Desktop!2fframeworkPages!2fframeworkpage!2fcom.sap.portal.innerpage?windowId=" + this.wid, null, null, null, this);			 
			break;
		case this.SET_PARAM:
			this.phase = this.ZERO_CONTACT;
			this.sid = this.getVariableValue("sap-ext-sid", data);		
			var location = "http://" + this.domain + "/webdynpro/dispatcher/sap.com/pb/PageBuilder?";
			location += "sap-ext-sid=" + this.sid + "&sap-wd-cltwndid=" + this.wid + 
				"&PagePath=pcd%3Aportal_content%2Fcom.bsnl.BSNL%2Fcom.bsnl.BSNL2%2Fcom.bsnl.roles%2Fcom.bsnl.BSNLRole01%2Fcom.bsnl.java_bsnl_com_wd_icss_com_bsnl_icss_applications_ICSSApp&sap-wd-app-namespace=zzzz&sap-ep-version=7.00.201010270856&sap-locale=en&sap-accessibility=&sap-rtl=&sap-cssversion=7.0.23.2.0&sap-epcm-guid=4B47780416A083FBFEB707E7A9200B41469CD8ECEC93D80A03A5E770C460F2E2&com.sap.portal.reserved.wd.pb.restart=false&DynamicParameter=";
			this.httpUtils2.talkToServer(location, null, null, null, this);
			break;
		case this.ZERO_CONTACT:			
			this.phase = this.GET_SUPPLIES;
			this.secureId = this.getVariableValue("sap-wd-secure-id", data);			
			var location = "http://" + this. domain + "/webdynpro/dispatcher/sap.com/pb/PageBuilder";
			location += "?sap-wd-cltwndid=" + this.wid;
			location += "&sap-wd-secure-id=" + this.secureId;
			location += "&sap-ext-sid=" + this.sid;
			location += "&sap-wd-norefresh=X&sap-ep-version=7.00.201010270856&sap-cssversion=7.0.23.2.0&sap-epcm-guid=4B47780416A083FBFEB707E7A9200B41469CD8ECEC93D80A03A5E770C460F2E2&DynamicParameter=&com.sap.portal.reserved.wd.pb.restart=false&PagePath=pcd%3Aportal_content%2Fcom.bsnl.BSNL%2Fcom.bsnl.BSNL2%2Fcom.bsnl.roles%2Fcom.bsnl.BSNLRole01%2Fcom.bsnl.java_bsnl_com_wd_icss_com_bsnl_icss_applications_ICSSApp&sap-accessibility=&sap-locale=en&sap-wd-app-namespace=zzzz&sap-rtl=";
			var pData = "sap-ep-version=7.00.201010270856&eventQueue=LINKCLICK%EE%80%82event%EE%80%82event%EE%80%82ControlId%EE%80%82aaab.NavigationView.LinkToAction38%EE%80%82CurrentFocusId%EE%80%82%EE%80%82time_parse%EE%80%827%EE%80%82time_applyInnerHTML%EE%80%827%EE%80%82time_scriptBeforeHTML%EE%80%820%EE%80%82time_scriptAfterHTML%EE%80%824%EE%80%82Id%EE%80%82aaab.NavigationView.LinkToAction38";
			this.httpUtils2.talkToServer(location, pData, null, null, this);			
			break;			
		case this.GET_SUPPLIES:		
			this.setState(this.LOGGEDIN);
			this.parseBSNLUsageData_ndc(data);
	}
}; 

DFox.prototype.parseBSNLUsageData_ndc = function (data) {
	var cProps = new Object();
	
	var downData = this.getNDCData("7", data);
	var upData = this.getNDCData("8", data);
	var totalData = this.getNDCData("9", data);
	var chargeData = this.getNDCData("13a", data);
	var hours = this.getNDCData("10", data);
	
	var td = parseFloat(downData);
	if (!isNaN(td)) cProps["dtR"] = td * 1024;
	
	td = parseFloat(upData);
	if (!isNaN(td)) cProps["dtS"] = td * 1024;
	
	td = parseFloat(totalData);
	if (!isNaN(td)) cProps["dtX"] = td * 1024;
	
	td = parseFloat(chargeData);
	if (!isNaN(td)) cProps["dcX"] = cProps["dnX"] = td * 1024;
	
	cProps["ntH"] = hours;
	
	var temp = new Date();
	cProps["stT"] = this.formatTime(temp);
	cProps["ntC"] = temp.getTime();
	
	var tProps = this.completeProgressCalculations(cProps["dcX"], cProps["ntC"], this.dateString);
	var prop;
	for (prop in tProps) {
		cProps[prop] = tProps[prop];
	}
	var eqString = this.createEquateString("this.activeDateString", this.dateString, "this.cProps", cProps);
	var eqString2 = this.createEquateString("this.cached", false);
	this.cacheDetails(eqString);
	this.push(this.combineEquateStrings(eqString, eqString2));
	this.refreshUI();
};


DFox.prototype.getVariableValue = function(lookfor, data) {
	var lf = 'name\\s*=\\s*"' + lookfor + '"\\s+value\\s*=\\s*"([^"]*)"';
	var re = new RegExp(lf, 'gi');
	var result = re.exec(data);
	if (result) {
		return result[1];
	}
	return null;
};

DFox.prototype.getNDCData = function(id, data) {
	var exp = new RegExp('<span\\s+id="aaab.BBUsageView.TextView' + id + '"[^>]+>([^<]*)</span>', 'gi');
	var result = exp.exec(data);
	if (result) {
		return result[1];		
	}
	return null;
};

DFox.prototype.bsnlResponseHandler_databsnlin = function(data) {
	switch(this.phase) {
		case this.FIRST_CONTACT:
			//get the address we can use to login
			var location = this.execRE(datafox_constants.logFrmExp, data);
			if (!location) {
				this.setState(this.ERROR, "Usage data N/A");
				return;
			}
			var pData = "wps.portlets.userid=" + this.username + "&password=" + this.password + "&ns_7_CGAH47L008DF802B56IQ0D00E0__login=Log+in";
			this.phase = this.LOGIN;
			this.httpUtils.talkToServer("http://" + this.domain + location[1], pData, null, null, this);
			break;
		case this.LOGIN:
			if (data.indexOf("Login failed") != -1) {
				this.setState(this.ERROR);
				break;
			}
			var details = this.execRE(datafox_constants.servicesExp, data);
			if (!details) {
				this.setState(this.ERROR, "Usage data N/A");
				return;
			}
			this.phase = this.SET_PARAMS;
			this.httpUtils.talkToServer("http://" + this.domain + details[1], pData, null, null, this);
			break;
		case this.SET_PARAMS:
			var rLocation = this.execRE(datafox_constants.searchFrmExp, data);
			if (!rLocation) {
				this.setState(this.ERROR, "Usage data N/A");
				return;
			}
			var pData = "hid_UsageSrchForm_ServiceNameOrAccountNumber=service&cmb_UsageSrchForm_ServiceName=Broadband&cmb_UsageSrchForm_AccountNumber=0&cmb_UsageSrchForm_UsageStatus=1&txt_UsageSrchForm_FromDate=&txt_UsageSrchForm_ToDate=";
			this.phase = this.GET_NEXT_PAGE;
			this.httpUtils.talkToServer("http://" + this.domain + rLocation[1], pData, null, null, this);
			break;
		case this.GET_NEXT_PAGE:
			var rLocation = this.execRE(datafox_constants.searchFrmExp, data);
			var id = this.execRE(datafox_constants.bbIdExp, data);
			if (!id || !rLocation) {
				this.setState(this.ERROR, "Usage data N/A");
				return;
			}

			//set the date range up correctly
			var parts = this.getYearMonth(this.dateString);
			var year = parts[0];
			var month = parts[1];
			var now = new Date();
			var dString = datafox_constants.month[month] + '-' + year + '-';
			if (month == now.getMonth()) {
				dString += "UnBilled";
			} else {
				dString += "Billed";
			}

			var pData = "hid_UsageSrchForm_ServiceNameOrAccountNumber=service&cmb_UsageSrchForm_ServiceName=Broadband&cmb_UsageSrchForm_ServiceUserId=" + encodeURIComponent(id[1]) +
			"&cmb_UsageDetailsSearch_UsgMnth=" + dString + "&imageBtn_UsageDetails_Search.x=51&imageBtn_UsageDetails_Search.y=5";
			this.openURL = "http://" + this.domain + rLocation[1];
			this.phase = this.GET_SUPPLIES;
			this.httpUtils.talkToServer(this.openURL, pData, null, null, this);
			break;
		case this.GET_SUPPLIES:
			this.setState(this.LOGGEDIN);
			this.parseBSNLUsageData_bbservice(data);
			break;
	}
};

DFox.prototype.parseBSNLUsageData_bbservice = function (aData) { //also handles the new data.bsnl.in portal
	//get "charged" usage
	var normal = this.getTableInfo(aData, 'Total Down', 0);
	if (!normal) return;

	//calculate totals
	var cProps = new Object();
	var temp = new Date();
	cProps["dtR"] = normal[0];
	cProps["dtS"] = normal[1];
	cProps["dtX"] = cProps["dtS"] + cProps["dtR"];
	cProps["dcX"] = cProps["dnX"] = normal[2];
	cProps["stT"] = this.formatTime(temp);
	cProps["ntC"] = temp.getTime();
	if (normal[3]) cProps["ntH"] = normal[3];

	var tProps = this.completeProgressCalculations(cProps["dcX"], cProps["ntC"], this.dateString);
	var prop;
	for (prop in tProps) {
		cProps[prop] = tProps[prop];
	}
	var eqString = this.createEquateString("this.activeDateString", this.dateString, "this.cProps", cProps);
	var eqString2 = this.createEquateString("this.cached", false);
	this.cacheDetails(eqString);
	this.push(this.combineEquateStrings(eqString, eqString2));
	this.refreshUI();
};


DFox.prototype.getTableInfo = function(aData, marker, sIdx) {
	var i0 = aData.indexOf(marker, sIdx);

	//check to see if anything was found...
	if (i0 < 0) {
		this.log("Could not find record summary table");
		this.setState(this.ERROR, "Usage data N/A");
		return false;
	}

	var databsnlin = (this.bsnlPortalType == 2);
	var table = databsnlin ? "TABLE" : "table";
	var tr = databsnlin ? "TR" : "tr";

	var i1 = aData.lastIndexOf('<' + table, i0);		//go backwards to find the table element
	var i2 = aData.indexOf('</' + table + '>', i1);			//find the point at which it ends

	aData = (aData.substring(i1, i2 + 8)).replace(/nowrap/gi, '').replace(/<BR>/gi, '').replace(/&nbsp;/gi, ''); //correct XML errors

	var parser = new DOMParser();
	var dom;
	try {
		dom = parser.parseFromString(aData, "text/xml");
	} catch (e) {
		this.log(e);
		this.setState(this.ERROR, "Usage data N/A");
		return false;
	}

	if (dom.firstChild.nodeName == "parsererror") {
		this.log("Could not parse summary table: " + dom.firstChild.textContent);
		this.setState(this.ERROR, "Usage data N/A");
		return false;
	}

	var results = dom.evaluate("//" + tr, dom, null, XPathResult.ANY_TYPE, null);
	results.iterateNext(); //skip the first header row
	var rows = results.iterateNext();
	var download = 0;
	var upload = 0;
	var charged = 0;
	try {
		var downCol, upCol, chargeCol;
		var multiplier;
		if (databsnlin) {
			downCol = "TD[4]";
			upCol = "TD[5]";
			chargeCol = "TD[8]";
			multiplier = datafox_constants.GiB_2_KiB; //value is in GB, convert to KB
		} else {
			downCol = "td[7]";
			upCol = "td[9]";
			chargeCol = "td[12]";
			multiplier = 1;
		}
		var down = dom.evaluate(downCol, rows, null, XPathResult.STRING_TYPE, null).stringValue;
  		var up = dom.evaluate(upCol, rows, null, XPathResult.STRING_TYPE, null).stringValue;
  		var charge = dom.evaluate(chargeCol, rows, null, XPathResult.STRING_TYPE, null).stringValue;
  		var duration = databsnlin ? dom.evaluate("TD[7]", rows, null, XPathResult.STRING_TYPE, null).stringValue : null;
  		if (duration) duration = duration.replace(/^\s+|\s+$/g, '') //trim!

  		var temp = parseFloat(down);
  		if (!isNaN(temp)) download = temp * multiplier;

  		temp = parseFloat(up);
  		if (!isNaN(temp)) upload = temp * multiplier;

  		temp = parseFloat(charge);
  		if (!isNaN(temp)) charged = temp * multiplier;
  	} catch (e) {
  		this.log(e);
  	}
	return [download, upload, charged, duration];
};

DFox.prototype.bsnlResponseHandler_sancharnet = function(data) {
	switch(this.phase) {
		case this.FIRST_CONTACT:
			this.phase = this.LOGIN;
			var pData = "username=" + this.username + "&password=" + this.password;
			this.httpUtils.talkToServer("http://" + this.domain + "/secu/myportallogin.jsp", pData, null, null, this);
			break;
		case this.LOGIN:
			this.phase = this.SET_PARAMS;
			//if the url contains loginerr...we have a problem
			if (data.indexOf("loginerr") != -1) {
			this.log("Site error");
				this.setState(this.ERROR);
				break;
			}
			datafox_constants.keyExp.lastIndex = 0;
			var keyA = datafox_constants.keyExp.exec(data);
			if (!keyA || keyA.length < 2) {
				this.log("Could not obtain key");
				this.setState(this.ERROR);
				break;
			}
			this.key = keyA[1];
			this.httpUtils.talkToServer("http://" + this.domain + ":80/myportal/loginsetparameter.jsp?key="+this.key, null, null, null, this);
			break;
		case this.SET_PARAMS:
			this.phase = this.GET_SUPPLIES;
			var gData = "key="+this.key+"&queryDate=" + this.dateString + "&username="+this.username+"&ispId=&idAndName=";
			var openURL = "http://" + this.domain + "/myportal/serviceResults.jsp?" + gData;
			this.push(this.createEquateString("this.openURL", openURL));
			this.httpUtils.talkToServer(openURL, null, null, null, this);
			break;
		case this.GET_SUPPLIES:
			this.setState(this.LOGGEDIN);
			this.parseBSNLUsageData(data);
			break;
	}
};

DFox.prototype.parseBSNLUsageData = function (aData) {
	this.log("Parsing usage data");
	//initialize all to N/A
    //dataFox.tSend = dataFox.tRecv = dataFox.tXfer = dataFox.tHours = dataFox.nXfer = "N/A";
    var upload, download, hours, xfer, nxfer;
    upload = download = hours = xfer = nxfer = "N/A";

    var temp = aData.split("Total Send Volume", 2);
    datafox_constants.dataExp.lastIndex = 0;
    if (!temp || temp.length < 2) {
    	this.setState(this.ERROR, "Usage data N/A");
    	return;
    }

    var result = temp[1].match(datafox_constants.dataExp);

    if (!result || result.length < 2) {
    	this.setState(this.ERROR, "Usage data N/A");
    	return;
    }

    datafox_constants.numExp.lastIndex = 0;
    result = result[1].match(datafox_constants.numExp);

	//values always stored in KB
	try {
		upload = parseFloat(result[0].substring(1)) * datafox_constants.GiB_2_KiB; //tS
	    download = parseFloat(result[1].substring(1)) * datafox_constants.GiB_2_KiB; //tR
	    xfer = parseFloat(result[2].substring(1)) * datafox_constants.GiB_2_KiB; //tX
	    hours = result[3].substring(1); //tH
	    nxfer = parseFloat(result[4].substring(1)) * datafox_constants.GiB_2_KiB; //nX
	} catch (e) {
		this.log(e);
	}

	var temp = new Date();
	var cProps = new Object();
	cProps["dtS"] = upload;
	cProps["dtR"] = download;
	cProps["dtX"] = xfer;
	cProps["dcX"] = cProps["dnX"] = nxfer;
	cProps["ntH"] = hours;
	cProps["stT"] = this.formatTime(temp);
	cProps["ntC"] = temp.getTime();

	var tProps = this.completeProgressCalculations(nxfer, cProps["ntC"], this.dateString);
	var prop;
	for (prop in tProps) {
		cProps[prop] = tProps[prop];
	}
	var eqString = this.createEquateString("this.activeDateString", this.dateString, "this.cProps", cProps);
	var eqString2 = this.createEquateString("this.cached", false);
	this.cacheDetails(eqString);
	this.push(this.combineEquateStrings(eqString, eqString2));
	this.refreshUI();
};

DFox.prototype.formatTime = function(time) {
	var hours = time.getHours();
	var min = time.getMinutes();
	var sec = time.getSeconds();
	var date = time.getDate();
	if (hours < 10) hours = "0" + hours;
	if (min < 10) min = "0" + min;
	if (sec < 10) sec = "0" + sec;
	if (date < 10) date = "0" + date;
	return  hours + ":" + min + ":" + sec + " " + datafox_constants.dow[time.getDay()] + ", " + datafox_constants.month[time.getMonth()] + " " + date;
};

DFox.prototype.daysInMonth = function(year, month) {
	return 32 - new Date(year, month, 32).getDate();
};

DFox.prototype.getYearMonth = function(dateString) {
	var parts = dateString.split("-");
	var year = parts[0];
	var mon = parseInt(parts[1], 10) - 1; //radix is important, values are 0 padded and would be parsed as octal otherwise
	return [year, mon];
};
