/*
 	This file is part of DataFox, a bandwidth usage monitor for
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2011 Brian Fernandes <infernalproteus@gmail.com>

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/
//constants

function datafox_constants() {
}

datafox_constants.XUL_NS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
datafox_constants.month = new Array("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");
datafox_constants.month_full = new Array("January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December");
datafox_constants.dow = new Array("Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday");
datafox_constants.MAX_TT_LINES = 10;
datafox_constants.ROUND_TO = 1000;
datafox_constants.SEPARATOR = ",;,";
datafox_constants.GiB_2_KiB = 1024 * 1024;
datafox_constants.MIDNIGHT = -19800000; //1st Jan 1970, 0000 IST
datafox_constants.EIGHTAM = 9000000; //1st Jan 1970, 0800 IST

function DFox() {
	this.name = "chrome://datafox/";
	this.dfPrefs = new datafox_prefs();
	this.httpUtils = new datafox_http();
	this.httpUtils2 = new datafox_http("Mozilla/5.0 (iPhone; U; CPU iPhone OS 3_0 like Mac OS X; en-us) AppleWebKit/528.18 (KHTML, like Gecko) Version/4.0 Mobile/7A341 Safari/528.16X-Hello");
	this.dfPrefs.initPrefs();
	this.dateString = this.getMonthString(new Date(), 0);
	datafox_PMgr.init();

	this.windowManager = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	try {
		this.sandbox = Components.utils.Sandbox("http://www.thegoan.com/datafox");
	} catch (e) {
		this.log(e);
	}

	try {
		var info = Components.classes["@mozilla.org/xre/app-info;1"].getService(Components.interfaces.nsIXULAppInfo);
		this.ff3 = parseInt(info.version.charAt(0)) >= 3;
	} catch (e) {
		this.ff3 = true;
		this.log(e);
	}

	this.log("Firefox 3: " + this.ff3);

	this.cProps = new Object();
	this.descCount = 0;
	this.descArray = new Array();
	for (var i = 0; i < datafox_constants.MAX_TT_LINES; i++) this.descArray[i] = document.createElement("description");

	//phase constants
	this.ZERO_CONTACT = 0;
	this.FIRST_CONTACT = 1;
	this.LOGIN = 2;
	this.SET_PARAMS = 3;
	this.GET_SUPPLIES = 4;
	this.GET_NEXT_PAGE = 5;

	//state constants
	this.ERROR = 0;
	this.INACTIVE = 1;
	this.LOGGINGIN = 2;
	this.LOGGEDIN = 3;

	//message constants
	this.SYNCH_REQ = 0;
	this.SYNCH = 1;
	this.SET_STATE = 2;
	this.EVAL = 3;
	this.REFRESH_UI = 4;
}

DFox.prototype.log = function(message) {
	if (!this.dfPrefs.getBoolPref(this.dfPrefs.DEBUG)) return;

	if (this._logService == null) {
	        this._logService = Components.classes['@mozilla.org/consoleservice;1'].getService();
	        this._logService.QueryInterface(Components.interfaces.nsIConsoleService);
	}

	this._logService.logStringMessage("datafox: " + message);
};


DFox.prototype.onLoad = function() {
	this.log("DataFox version 1.9.5");

	//widgets...
	this.ttMessage = document.getElementById("df-statusbar-tooltip-message");
	this.ttBold = document.getElementById("df-statusbar-tooltip-username");
	this.statusBar = document.getElementById("df-statusbar");
	this.progressMeter = document.getElementById("df-progress-meter");
	this.progressBar = document.getAnonymousElementByAttribute(this.progressMeter, "class", "progress-bar");
	this.ttCustom = document.getElementById(this.ff3 ? "df-statusbar-tooltip-custom-FF3" : "df-statusbar-tooltip-custom");
	this.dfLogout = document.getElementById("df-menuitem-logout");
	this.dfUpdate = document.getElementById("df-menuitem-update");
	this.dfBrowser = document.getElementById("df-menuitem-browser");
	this.dfBrowserPopup = document.getElementById("df-menuitem-browser-popup");
	this.dfBrowserAll = document.getElementById("df-menuitem-allmonths");
	this.dfMonth = document.getElementById("df-menuitem-month");
	this.dfContextMenu = document.getElementById("df-status-context");
	this.dfContextSeparator = document.getElementById("df-menuitem-separator");
	this.dfStatusStack = document.getElementById("df-status-stack");

	this.observerService = Components.classes["@mozilla.org/observer-service;1"].getService(Components.interfaces.nsIObserverService);

	//are there any other observers around?
	var enumeration = this.observerService.enumerateObservers(this.name);
	var existing = enumeration && enumeration.hasMoreElements();
	var firstBrowser = existing ? enumeration.getNext() : null;

	//add observer
    this.observerService.addObserver(this, this.name, false);
	//add preference observer
	this.dfPrefs.dfPrefObserver.register();

	//rounding preference
	var places = this.dfPrefs.getIntPref(this.dfPrefs.DECIMAL_PLACES);
	datafox_constants.ROUND_TO = Math.pow(10, places);
	//sanity check
	if (isNaN(datafox_constants.ROUND_TO)) {
		datafox_constants.ROUND_TO = 1000;
	}

	//text color
	this.updateTextColor();

	//progress color
	this.updateProgressMeterUI();

	if (!existing) {
		//initialize to inactive state
		this.setState(this.INACTIVE);

		this.updateLoginDetails();

		if (this.dfPrefs.getBoolPref(this.dfPrefs.LOGIN_STARTUP)) {
			this.logIn();
		} else {
			this.showCachedDetails();
		}
	} else {
		//ask for synch up from first observer
		var observer = firstBrowser.QueryInterface(Components.interfaces.nsIObserver);
		observer.observe(this, this.name, this.SYNCH_REQ + "");
	}
};

DFox.prototype.showCachedDetails = function() {
	var cached = this.dfPrefs.getCharPref(this.dfPrefs.CACHE_DATA);
	if (cached) {
		this.cached = true;
		this.push(cached);
		this.setState(this.LOGGEDIN);
		this.refreshUI();
		return true;
	}
	return false;
};

DFox.prototype.cacheDetails = function(eqString) {
	eqString += "this.username=\"" + this.username + "\";this.dateString=\"" + this.dateString + "\";this.cached=true;";
	this.dfPrefs.setCharPref(this.dfPrefs.CACHE_DATA, eqString);
};

DFox.prototype.onUnload = function() {
	this.observerService.removeObserver(this, this.name);
	this.dfPrefs.dfPrefObserver.unregister();
};

DFox.prototype.update = function(prompt, aEvent) {
	 // on windows, right click calls this method
  	if (aEvent) {
  		if (aEvent.button == 2) return;
  		if (aEvent.ctrlKey) {
  			if (this.showCachedDetails()) return;
  		}
  	}
	if (this.insufficientLoginInfo()) {
		//insufficient data to log in
		if (prompt && this.windowManager.getMostRecentWindow("datafox:login") == null) {
			window.openDialog('chrome://datafox/content/login.xul', '_blank', 'chrome,centerscreen,resizable=no,dependant=yes');
		}
		return;
	}
	this.logIn();
};

DFox.prototype.openPrefs = function() {
	if (this.windowManager.getMostRecentWindow("datafox:preferences") == null) {
		window.openDialog('chrome://datafox/content/preferences.xul', '_blank');
	}
}

DFox.prototype.insufficientLoginInfo = function() {
	return 	(!this.username ||
			(!this.password && this.isp != 1) || //password not required for MTNL Bombay
			(!this.domain && this.isp == 0 && this.isOverrideActive()));
};

DFox.prototype.logout = function() {
	this.log("Logging out");
	this.setState(this.INACTIVE);
};

DFox.prototype.isOverrideActive = function() {
	return this.dfPrefs.getBoolPref(this.dfPrefs.OVERRIDE_DOMAIN);
};

DFox.prototype.updateLoginDetails = function() {
	var username = this.dfPrefs.getCharPref(this.dfPrefs.ACTIVE_USER);
	if (username) {
		this.password = datafox_PMgr.getSavedPassword(username);
	} else {
		this.password = null;
	}
	this.username = username;

	this.setISP(this.dfPrefs.getIntPref(this.dfPrefs.ACTIVE_ISP));
	this.log("ISP is " + this.isp);
	if (this.isp == 0 && this.isOverrideActive()) {
		this.log("IP override active");
		this.domain = this.dfPrefs.getCharPref(this.dfPrefs.ACTIVE_DOMAIN);
	}
};

DFox.prototype.logIn = function() {
	//not enough info to go on
	if (this.insufficientLoginInfo()) {
		this.log("Not enough info to log in");
		return;
	}

	this.log("Logging in with account " + this.username);

	//continue with the login process
	//change icon
	this.setState(this.LOGGINGIN, this.dateString);

	switch(this.isp) {
		case 0:
			this.bsnlLogin();
			break;
		case 1:
			this.mtnlBOMLogin();
			break;
		case 2:
			this.mtnlDELLogin();
			break;
	}
};

DFox.prototype.responseHandler = function(data, request) {
	//cancelled request
	if (request && request.status == Components.results.NS_ERROR_ABORT) return;
	switch(this.isp) {
		case 0:
			this.bsnlResponseHandler(data);
			break;
		case 1:
			this.mtnlBOMResponseHandler(data);
			break;
		case 2:
			this.mtnlDELResponseHandler(data);
			break;
	}
};

DFox.prototype.setState = function(state, eMsg) {
	this.observerService.notifyObservers(this,
										this.name,
										this.SET_STATE + datafox_constants.SEPARATOR + state + (eMsg ? (datafox_constants.SEPARATOR + eMsg) : ""));
};

DFox.prototype.observe = function(subject, topic, data) {
	var parts = data.split(datafox_constants.SEPARATOR);
	var msgType = parseInt(parts[0]);

	switch (msgType) {
		case this.SYNCH_REQ:
			this.log("SYNCH REQUEST RECEIVED");
			//collect all the data required
			var evString = this.createEquateString("this.username", this.username,
									"this.password", this.password,
									"this.domain", this.domain,
									"this.cProps", this.cProps,
								  	"this.activeDateString", this.activeDateString,
								  	"this.openURL", this.openURL,
								  	"this.cached", this.cached,
								  	"this.isp", this.isp,
								  	"this.state", this.state
								  	);

			this.observerService.notifyObservers(this, this.name, this.SYNCH + datafox_constants.SEPARATOR + evString);
//			subject.observe(this, this.name, this.SYNCH + datafox_constants.SEPARATOR + evString);
			break;
		case this.SYNCH:
			this.log("synching");
			this.safeExec(parts[1]);
			this.internalSetState(this.state);
			this.updateStrings();
			break;
		case this.SET_STATE:
			this.internalSetState(parseInt(parts[1]), ((parts.length == 3) ? parts[2] : null));
			break;
		case this.EVAL:
			this.safeExec(parts[1]);
			break;
		case this.REFRESH_UI:
			this.updateStrings();
			break;
	}
};

DFox.prototype.safeExec = function(evalString) {
	Components.utils.evalInSandbox(evalString, this.sandbox);
	for (i in this.sandbox) {
		this[i] = this.sandbox[i];
		delete this.sandbox[i];
		//special cases
		if (i == "isp") {
			this.setISP(this.isp);
		}
	}
};

DFox.prototype.internalSetState = function(state, eMsg) {
	this.state = state;
	switch(state) {
		case this.ERROR:
			if (!eMsg) eMsg = "Error logging in, please check your username, password & domain";
			this.updateUI("error",
							"DataFox: " + eMsg,
							"",
							true,
							true,
							false,
							true);
			this.username = null;
			this.statusBar.label = "";
			break;
		case this.INACTIVE:
			this.updateUI("inactive",
							"Click to load",
							"",
							false,
							true,
							false,
							true);
			this.username = null;
			this.statusBar.label = "";
			this.cancelComm(this.httpUtils);
			this.cancelComm(this.httpUtils2);	
			break;
		case this.LOGGINGIN:
			this.updateUI("loading",
							"Loading " + eMsg,
							"",
							false,
							true,
							true,
							false);
			this.activeDateString = null;
			break;
		case this.LOGGEDIN:
			this.updateUI("active",
							"Logged in as ",
							this.username,
							false,
							false,
							false,
							false);
			break;
	}
};

DFox.prototype.cancelComm = function(httpUtils) {
	if (httpUtils.httpChannel != null) {
		try {
			httpUtils.httpChannel.cancel(Components.results.NS_ERROR_ABORT);
		} catch (e) {
			
		}
	}
};

DFox.prototype.updateUI = function(status, message, bMessage, error, collapsed, uState, lState) {
	this.statusBar.setAttribute("df-status", status);
	this.ttMessage.value = message;
	this.ttBold.value = bMessage;
	this.ttMessage.setAttribute("error", error);
	this.ttCustom.collapsed = collapsed;
	this.dfUpdate.setAttribute("disabled", uState);
	this.dfLogout.setAttribute("disabled", lState);
	this.dfBrowser.setAttribute("disabled", uState);
	this.dfMonth.setAttribute("disabled", uState);// || (this.isp == 1));
	//remove the progress bar if necessary
	if (lState && this.progressMeter.parentNode) {
		this.dfStatusStack.removeChild(this.progressMeter);
	}
};

DFox.prototype.createCustomTooltip = function(parent, inString) {
	if (this.ff3) {
		this.ttCustom.textContent = inString;
		return;
	}

	var parts = inString.split("\n");
	var i;
	for (i = 0; i < parts.length && i < datafox_constants.MAX_TT_LINES; i++) {
		parent.appendChild(this.descArray[i]);
		this.descArray[i].value = parts[i];
	}

	while (i < this.descCount) {
		parent.removeChild(this.descArray[i++]);
	}

	this.descCount = parts.length;
};

DFox.prototype.setISP = function(value) {
	this.isp = value;
	switch(this.isp) {
		case 1:
			//Bombay
			if (this.dfMonth.parentNode) {
				this.dfContextMenu.removeChild(this.dfMonth);
			}
			this.dfBrowserAll.label = "All Months";
			break;
		case 2:
			//Delhi
			if (this.dfMonth.parentNode) {
				this.dfContextMenu.removeChild(this.dfMonth);
			}
			this.dfBrowserAll.label = "Last Month";
			break;
		case 0:
			var subType = 0;
			if (this.isOverrideActive()) {
				var domain = this.dfPrefs.getCharPref(this.dfPrefs.ACTIVE_DOMAIN);
				if (this.isSelfcare(domain)) {
					subType = 3;
				}
			}
			if (subType != 3) {
				if (!this.dfMonth.parentNode) {
					this.dfContextMenu.insertBefore(this.dfMonth, this.dfContextSeparator);					
				}				
			} else {
				if (this.dfMonth.parentNode) {
					this.dfContextMenu.removeChild(this.dfMonth);
				}
			}
			this.dfBrowserAll.label = "All Months";			
			break;
	}
};

DFox.prototype.createMonthMenu = function(menupopup) {
	var date = new Date();
	date.setDate(1);
	var children = menupopup.childNodes;
   	for (var i = 0; i < children.length; i++) {
   		children[i].label = datafox_constants.month[date.getMonth()] + " " + new String(date.getFullYear()).substring(2);
   		var dString = this.getMonthString(date, i);
		children[i].setAttribute("oncommand", "dataFox.selectMonth('" + dString + "')");
		children[i].setAttribute("checked", (dString == this.activeDateString));
   		date.setMonth(date.getMonth()-1);
   	}
};

DFox.prototype.openInBrowser = function(openRecords) {
	var url;
	if (this.isp == 0) {
		//bsnl
		if (!openRecords || this.bsnlPortalType >= 2) {
			url = this.openURL;
		} else if(this.bsnlPortalType == 1) {
			url = "http://" + this.domain + "/UsageDetails.jsp";
		} else {
			url = "http://" + this.domain + "/myportal/serviceRecords.jsp";
		}
	} else if (this.isp == 1) {
		this.mtnlBOMOpenInBrowser(openRecords);
		return;
	} else if (this.isp == 2) {
		this.mtnlDELOpenInBrowser(openRecords);
		return;
	}
	this.httpUtils.openInBrowser(url);
};

DFox.prototype.selectMonth = function(dString) {
	this.dateString = dString;
	this.log("Selecting month: " + this.dateString);
	//this.statusBar.label = "Loading: " + dString;
	//this.setState(this.LOGGINGIN, this.dateString);
	this.update(true);
};

DFox.prototype.getMonthString = function(date, idx) {
	var mon = date.getMonth() + 1;
	return date.getFullYear() + "-" + (mon < 10 ? "0" : "") + mon;
}

DFox.prototype.isBetween12_8 = function(t1, t2) {
	if (t1.getDate() != t2.getDate()) return false;

	//set them to 1st Jan 1970, which is our reference date
	t1.setDate(1);
	t1.setMonth(0);
	t1.setYear(1970);

	t2.setDate(1);
	t2.setMonth(0);
	t2.setYear(1970);

	return (t1 >= datafox_constants.MIDNIGHT && t2 <= datafox_constants.EIGHTAM);
}


DFox.prototype.updateStrings = function() {
	//only if we're logged in
	if (this.state != this.LOGGEDIN) return;

	//round the digits
	var vArray = this.valuesToString();

	//var sString = "{tX} / {nX}";
    var sString = dataFox.dfPrefs.getCharPref(dataFox.dfPrefs.STATUS_STRING);
    this.statusBar.label = this.evalString(sString, vArray);

    //set the tooltip
	var tString = dataFox.dfPrefs.getCharPref(dataFox.dfPrefs.TOOLTIP_STRING);
	this.createCustomTooltip(this.ttCustom, this.evalString(tString, vArray));

	//update the progress bar
	this.updateProgressMeterUI();

	//update color on the Text bar
	this.updateTextColor();
};


DFox.prototype.completeProgressCalculations = function(used, mSecs, activeDateString) {
	var cProps = new Array();
	var limit = this.dfPrefs.getIntPref(this.dfPrefs.BANDWIDTH_LIMIT);
	if (limit > 0) {
		//var used = this.cProps["dcX"] / 1024; //value is in KiB, converting to MiB
		var allowedPercent = this.getAllowedUsePercent(mSecs, activeDateString);
		var allowedUse = limit * 1024 * allowedPercent / 100; //limit is MiB, converting to KiB
		var usedPercent = (100 * used) / (limit * 1024); //used is in KiB
		cProps["naP"] = allowedPercent;
		cProps["nuP"] = usedPercent;
		cProps["dsX"] = allowedUse - used;
		cProps["drX"] = (limit * 1024) - used;
	} else {
		cProps["naP"] = null;
		cProps["nuP"] = null;
		cProps["dsX"] = null;
	}
	return cProps;
};

DFox.prototype.updateProgressMeter = function() {
	var tProps = this.completeProgressCalculations(parseInt(this.cProps["dcX"]),
													parseInt(this.cProps["ntC"]),
													this.activeDateString);
	var prop;
	for (prop in tProps) {
		this.cProps[prop] = tProps[prop];
		this.log(this.cProps[prop]);
	}
	this.push(this.createEquateString("this.cProps", this.cProps));
	this.refreshUI();
};

DFox.prototype.getAllowedUsePercent = function(mSecs, activeDateString) {
	var gran = this.dfPrefs.getIntPref(this.dfPrefs.USAGE_GRANULARITY);
	//var time = new Date(parseInt(this.cProps["ntC"]));
	var time = new Date(mSecs);
	var days = this.daysInMonth(time.getYear(), time.getMonth());

	if (this.isp == 0) {
		//BSNL only
		var loggedMonth = parseInt(activeDateString.substr(5), 10) - 1;
		if (loggedMonth != time.getMonth()) {
			//fetching data for a past month, allowed is 100%
			return 100;
		}
	}

	if (gran == 0) {
		//daily
		var passedDays = time.getDate();
		return passedDays * 100 / days;
	} else {
		passedHours = time.getHours() + (time.getDate() - 1) * 24;
		return passedHours * 100 / (days * 24);
	}
};

DFox.prototype.updateProgressMeterUI = function() {
	var barOn = this.dfPrefs.getBoolPref(this.dfPrefs.PROG_ENABLE);
	if (barOn && this.cProps["nuP"] != null) {
		//bar must be visible
		if (!this.progressMeter.parentNode) {
			this.log("enabling progressMeter");
			this.progressMeter = document.createElementNS(datafox_constants.XUL_NS, "progressmeter");
			this.progressMeter.setAttribute("id", "df-progress-meter");
			this.progressMeter.setAttribute("mode", "determined");
			this.dfStatusStack.insertBefore(this.progressMeter, this.statusBar);
			this.progressBar = document.getAnonymousElementByAttribute(this.progressMeter, "class", "progress-bar");
		}
		//update bar details
		this.progressMeter.value = this.cProps["nuP"];
		var cVar = this.getCool();
		this.progressBar.style.background = this.dfPrefs.getCharPref("datafox.prog" + cVar);
		this.progressMeter.style.background = this.dfPrefs.getCharPref("datafox.bkgnd" + cVar);
		this.progressMeter.setAttribute("bcolor", this.dfPrefs.getBoolPref(this.dfPrefs.BKGND_ENABLE));
	} else {
		this.log("disabling progressMeter");
		//bar must be invisible
		if (this.progressMeter.parentNode) {
			this.dfStatusStack.removeChild(this.progressMeter);
		}
		this.progressMeter.value = 0;
	}
};

DFox.prototype.updateTextColor = function() {
	this.statusBar.style.color = this.dfPrefs.getCharPref("datafox.text" + (!this.cached ? this.getCool() : ".cached"));
};

DFox.prototype.getCool = function() {
	var allowedPercent = this.cProps["naP"];
	var usedPercent = this.cProps["nuP"];
	return (allowedPercent == null || usedPercent == null || usedPercent <= allowedPercent) ? ".cool" : ".uncool";
};

DFox.prototype.evalString = function(sString, vArray) {
	sString = this.formatString(sString, vArray);
    var out;
    try {
   		out = Components.utils.evalInSandbox(sString, this.sandbox);
   	} catch (e) {
   		this.log(e);
   	}
   	this.log(typeof(out));
   	if (typeof(out) != "string") {
   		out = sString;
   	}
   	return out;
};

DFox.prototype.formatString = function(sString, array) {
	var prop;
	for (prop in array) {
		if (array[prop] != null) {
			sString = sString.replace(new RegExp("{" + prop.substring(1) + "}", "g"), array[prop]);
		}
	}
	sString = sString.replace(/{YM}/g, this.dateString);
	return sString.replace(/{.{2}}/g, "N/A");	//replace any variables not found with "N/A"
};

DFox.prototype.createEquateString = function() {
	var evalString = "";
	for (var i = 0; i < arguments.length; i+=2) {
		var type = typeof(arguments[i+1]);
		if (type == "object") {
			//reset the object
			evalString += arguments[i] + "= new Object();";
			var avar;
			for (avar in arguments[i+1]) {
				evalString += arguments[i] + "['" + avar + "']=" + this.wrap(arguments[i+1][avar]) + ";";
			}
		} else if (type != "undefined")  {
		 	evalString += arguments[i] + "=" + this.wrap(arguments[i+1]) + ";";
		}
	}
	return evalString;
};

DFox.prototype.combineEquateStrings = function(s1, s2) {
	return s1 + s2;
}

DFox.prototype.wrap = function(input) {
	return (typeof(input) == "string") ? ("'" + input + "'") : input;
}

DFox.prototype.push = function(dataString) {
	this.observerService.notifyObservers(this, this.name, this.EVAL + datafox_constants.SEPARATOR + dataString);
};

DFox.prototype.refreshUI = function() {
	this.observerService.notifyObservers(this, this.name, this.REFRESH_UI);
};

DFox.prototype.valuesToString = function() {
	var mb = this.dfPrefs.getIntPref(this.dfPrefs.DATA_UNIT);
	var array = new Object();
	var prop;
	for (prop in this.cProps) {
		if (this.cProps[prop] == null) continue;
		if (prop.length == 3 && prop[0] == "d") {
			//we've got to round and convert this value, it's a data value
			array[prop] = this.RAC(this.cProps[prop], mb);
		} else if (prop[prop.length-1] == "P") {
			array[prop] = Math.round(datafox_constants.ROUND_TO * this.cProps[prop]) / datafox_constants.ROUND_TO;
		} else {
			array[prop] = this.cProps[prop];
		}
	}
	return array;
};

DFox.prototype.RAC = function(value, mb) {
	value /= 1024; 			//convert to MiB
	if (!mb) value /= 1024; //convert to GiB
	return Math.round(datafox_constants.ROUND_TO * value) / datafox_constants.ROUND_TO;
};

DFox.prototype.execRE = function(re, data) {
	re.lastIndex = 0;
	return re.exec(data);
};

var dataFox = new DFox();

window.addEventListener("load", function() {dataFox.onLoad();}, false);
window.addEventListener("unload", function() {dataFox.onUnload();}, false);
