/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2011 Brian Fernandes <infernalproteus@gmail.com>

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/

function datafox_http(agent) {
	this.httpChannel = null;
	this.channel = null;	
	this.agent = agent;
}

datafox_http.prototype.talkToServer = function(aURL, aPostData, aCookieData, aReferrer, dfInstance, redirect, timeout, mLength) {
  dataFox.log(aURL);
  // the IO service
  var ioService = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
  // create an nsIURI
  var uri = ioService.newURI(aURL, null, null);
  //nsIInputStream
  var uploadStream = Components.classes["@mozilla.org/io/string-input-stream;1"]
    .createInstance(Components.interfaces.nsIStringInputStream);
  if (aPostData)
    uploadStream.setData(aPostData, aPostData.length);

  this.channel = ioService.newChannelFromURI(uri);
  // get a httpchannel and make it a post
  this.httpChannel = this.channel.QueryInterface(Components.interfaces.nsIHttpChannel);
  //httpChannel.setRequestHeader('Accept-Encoding', 'gzip, deflate', false);
//  httpChannel.setRequestHeader('Range', 'bytes=0-8000', false);
  if (this.agent) {
	  this.httpChannel.setRequestHeader("User-Agent", this.agent, false);
  }
  // set a referrer
  if (aReferrer) {
    var referrerUri = ioService.newURI(aReferrer, null, null);
    this.httpChannel.referrer = referrerUri;
  }
  if (aPostData) {
    var uploadChannel = this.channel.QueryInterface(Components.interfaces.nsIUploadChannel);
    uploadChannel.setUploadStream(uploadStream, "application/x-www-form-urlencoded", -1);

    // order important - setUploadStream resets to get/put
    this.httpChannel.requestMethod = "POST";
  }
 if (aCookieData){
    this.httpChannel.setRequestHeader("Cookie", aCookieData, false);
    /*for (var run = 0; run < aCookieData.length; run++){
      httpChannel.setRequestHeader("Cookie", aCookieData[run], true);
    }*/
  }
  
  if (dfInstance) {
	  var observer;
	  if (!redirect) {
	  	observer = new this.httpObserver(dfInstance, mLength, this);
	  	/*if (timeout) {
	  		setTimeout("httpChannel.cancel(Components.results.NS_ERROR_ABORT);", timeout);
	  	}*/
	  } else {
	  	observer = new this.redirectObserver(dfInstance);
		this.channel.notificationCallbacks = observer;		  
	  }
	  return this.channel.asyncOpen(observer, null);
  } else {
  	  return this.channel.open();
  }	  
}


datafox_http.prototype.convertToPostData = function(dataString) {
	// POST method requests must wrap the encoded text in a MIME
	// stream
	if (!dataString) return null;
	
	const Cc = Components.classes;
	const Ci = Components.interfaces;
	var stringStream = Cc["@mozilla.org/io/string-input-stream;1"].
	                   createInstance(Ci.nsIStringInputStream);
	stringStream.setData(dataString, dataString.length);
	
	var postData = Cc["@mozilla.org/network/mime-input-stream;1"].
	               createInstance(Ci.nsIMIMEInputStream);
	postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
	postData.addContentLength = true;
	postData.setData(stringStream);
	
	return postData;
}


datafox_http.prototype.redirectObserver = function(dfInstance) {
	return ({
		newIP : "",
		onStartRequest : function (aRequest, aContext) {
	    },
	    onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
	    },
	    onStopRequest : function (aRequest, aContext, aStatus) {
	    	if (this.newIP) {
	    		dfInstance.redirectIP(this.newIP);			
	    	} else {
	    		//probably no internet connection or redirect failure
	    		dfInstance.log("Could not obtain server IP"); 
	    		dfInstance.setState(dfInstance.ERROR, "Error obtaining server IP. Check internet connection/use IP override");
	    	}
	    },    
		onChannelRedirect: function (aOldChannel, aNewChannel, aFlags) {
			this.newIP = aNewChannel.URI.host;
		},	  
		onRedirect : function (aOldChannel, aNewChannel) {
		},
		onProgress : function (aRequest, aContext, aProgress, aProgressMax) { 
		},
	  	onStatus : function (aRequest, aContext, aStatus, aStatusArg) { 
	  	},
		getInterface: function(aIID) {
		   return this;
	  	}
  	});
}


datafox_http.prototype.httpObserver = function(dfInstance, mLength, datafox_http) {
  return ({
    data : "",	
    length: 0,
	
    onStartRequest : function (aRequest, aContext) {
		this.data = "";
    },

    onDataAvailable : function (aRequest, aContext, aStream, aSourceOffset, aLength){
      var scriptableInputStream = 
        Components.classes["@mozilla.org/scriptableinputstream;1"]
          .createInstance(Components.interfaces.nsIScriptableInputStream);
      scriptableInputStream.init(aStream);

      this.data += scriptableInputStream.read(aLength);
      this.length += aLength;
      if (mLength && this.length > mLength) {
    	  dfInstance.log("Aborting transfer, fetch limit exceeded");
    	  datafox_http.httpChannel.cancel(Components.results.NS_ERROR_FAILURE);
      }
    },

    onStopRequest : function (aRequest, aContext, aStatus) {
		dfInstance.responseHandler(this.data, aRequest);
    },     
    
    cancel : function() {
    	dfInstance.log("cancelled");
    },

    QueryInterface : function(aIID) {
      if (aIID.equals(Components.interfaces.nsIStreamListener) ||
          aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
          aIID.equals(Components.interfaces.nsIAlertListener));
        return this;
      throw Components.results.NS_NOINTERFACE;
    }
  }
  );
}

datafox_http.prototype.getBrowserForTab = function(aTab){
  var panel = document.getElementById(aTab.linkedPanel);
  return panel ? 
  			(panel.firstChild.nextSibling ?  panel.firstChild.nextSibling
  										  : panel.firstChild)
  			 : null;
}

datafox_http.prototype.openInBrowser = function(url, postData) {
	var currentTab = getBrowser().mCurrentTab;
	var browser = this.getBrowserForTab(currentTab);
	//is the current tab blank?
	var reusable = (!browser.sessionHistory || browser.sessionHistory.count < 2 &&
		(!browser.currentURI || browser.currentURI.spec == 'about:blank') &&
		!currentTab.hasAttribute("busy"));
	if (reusable) {
		getBrowser().loadURIWithFlags(url, null, null, null, this.convertToPostData(postData));
	} else {
		getBrowser().loadOneTab(url, null, null, this.convertToPostData(postData));
	}
}
