/*
 	This file is part of DataFox, a bandwidth usage monitor for
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2008 Brian Fernandes <infernalproteus@gmail.com>

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/

var dataFox = window.opener.dataFox;
var dfPMgr = window.opener.datafox_PMgr;
var dfPrefs = dataFox.dfPrefs;
var usernameW;
var passwordW;
var savepwW;
var domainLW;
var domainW;
var overrideW;
var ispW;
var rowsW;
var overrideRowW;
var domainRowW;
var passwordRowW;
var storePasswordRowW;
var titleW;

var host = "chrome://datafox/";


function onLoad() {
	usernameW = document.getElementById("username");
	passwordW = document.getElementById("password");
	savepwW = document.getElementById("store-password");
	overrideW = document.getElementById("override-domain");
	domainLW = document.getElementById("domain-label");
	domainW = document.getElementById("domain");
	rowsW = document.getElementById("layout-rows");
	passwordRowW = document.getElementById("password-row");
	storePasswordRowW = document.getElementById("store-password-row");
	overrideRowW = document.getElementById("override-row");
	domainRowW = document.getElementById("domain-row");
	titleW = document.getElementById("login-title");
	ispW = document.getElementById("isp");

	//populate the user list
	populateUserList();

	//load username and password
	var lastUser = dfPrefs.getCharPref(dfPrefs.ACTIVE_USER);
	if (lastUser) {
		usernameW.value = lastUser;
		var password = dfPMgr.getSavedPassword(lastUser);
		if (password && password.length > 0) {
			passwordW.value = password;
		}
	}

	//load "save password"
	savepwW.checked = dfPrefs.getBoolPref(dfPrefs.SAVE_PASSWORD);

	//load domain details
	overrideW.checked = dfPrefs.getBoolPref(dfPrefs.OVERRIDE_DOMAIN);

	var domain = dfPrefs.getCharPref(dfPrefs.ACTIVE_DOMAIN);
	if (domain) {
		domainW.value = domain;
	}

	//load active isp
	var isp = dfPrefs.getIntPref(dfPrefs.ACTIVE_ISP);
	ispW.selectedIndex = isp;
	ispChanged(isp);

	overrideChanged();
}

function populateUserList() {
	var users = dfPMgr.getUserNames();
	for (var i = 0; i < users.length; i++) {
		usernameW.appendItem(users[i], users[i]);
	}
}

function ispChanged(selectedIndex) {
	var changed = false;
	switch (selectedIndex) {
		case 0:
			//BSNL
			titleW.setAttribute("bsnl", true);
			if (!overrideRowW.parentNode) {
				rowsW.appendChild(overrideRowW);
				rowsW.appendChild(domainRowW);
				changed = true;
			}
			break;
		case 1:
			//MTNL Bombay
			//falls through
		case 2:
			//MTNL Delhi
			titleW.setAttribute("bsnl", false);
			if (overrideRowW.parentNode) {
				rowsW.removeChild(overrideRowW);
				rowsW.removeChild(domainRowW);
				changed = true;
			}
			break;
	}
	if (changed) window.sizeToContent();
}

function overrideChanged() {
	domainW.disabled = domainLW.disabled = !overrideW.checked;
}

function loadPassword() {
	var password = dfPMgr.getSavedPassword(usernameW.value);
	if (password && password.length > 0) {
		passwordW.value = password;
	}
}

function onCancel() {
	dataFox.log("Login cancelled");
	dataFox.setState(dataFox.INACTIVE);
}

function onAccept() {
	dataFox.log('accepting');
	var isp = ispW.selectedIndex;

	var username = usernameW.value;

	var newBOMPortal = "";
	if (isp == 1) {
		//MTNL Bombay
		newBOMPortal = "this.newBOMPortal=" + ((username.indexOf("@") == -1) ? "false" : "true") + ";";
	}

	var evalString = newBOMPortal + "this.isp=" + isp + ";";
	dfPrefs.setIntPref(dfPrefs.ACTIVE_ISP, isp);

	evalString += "this.username='" + username + "';";
	dfPrefs.setCharPref(dfPrefs.ACTIVE_USER, usernameW.value);

	//manage the password store
	dfPMgr.managePassword(usernameW.value, passwordW.value, savepwW.checked);

	dfPrefs.setBoolPref(dfPrefs.SAVE_PASSWORD, savepwW.checked);
	evalString += "this.password='" + passwordW.value + "';";
	if (isp == 0) {
		dfPrefs.setCharPref(dfPrefs.ACTIVE_DOMAIN, domainW.value);
		dfPrefs.setBoolPref(dfPrefs.OVERRIDE_DOMAIN, overrideW.checked);
		if (overrideW.checked) {
//			dataFox.domain = domainW.value;
			evalString += "this.domain='" + domainW.value + "';";
		} else {
			evalString += "this.domain=null;";
		}
	} else {
		evalString += "this.domain=null;";
	}

	dataFox.push(evalString);
	dataFox.update(false);
	return true;
}
