/*
 	This file is part of DataFox, a bandwidth usage monitor for
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2009 Brian Fernandes <infernalproteus@gmail.com>

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/
datafox_constants.mbUsed = new RegExp("MB Used[\\s\\S]*?(\\d*\\.?\\d+)", "gi");
datafox_constants.bMBUsed = new RegExp("Current Month Usage[\\s\\S]*?(\\d*\\.?\\d+)", "gi");
datafox_constants.adslFree = new RegExp("ADSL MB[\\s\\S]*?(\\d*\\.?\\d+)", "gi");
datafox_constants.nightMB = new RegExp("NightMB[\\s\\S]*?(\\d*\\.?\\d+)", "gi");
datafox_constants.redirect = new RegExp("url=http://([^/]+)", "gi");

datafox_constants.bstrAccId = new RegExp("c_strAccountId=(\\w+)", "gi");

DFox.prototype.mtnlBOMLogin = function() {
	if (this.username.indexOf("@") == -1) {
		this.domain = "register.mtnl.net.in/mtnlWebInt"; //no redirection required for now
		this.newBOMPortal = false;
	} else {
		this.domain = "59.185.101.2:10080";
		this.newBOMPortal = true;
	}

	if (this.domain) {
		this.phase = this.FIRST_CONTACT;
		this.httpUtils.talkToServer("http://" + this.domain + "/jsp/customer/Login.jsp", null, null, null, this);
	} else {
		this.phase = this.ZERO_CONTACT;
		this.httpUtils.talkToServer("http://register.mtnl.net.in/", null, null, null, this);
	}
};

DFox.prototype.mtnlBOMResponseHandler = function(data) {
	if (!this.newBOMPortal) {
		this.mtnlBOMResponseHandler_old(data);
	} else {
		this.mtnlBOMResponseHandler_new(data);
	}
};

DFox.prototype.mtnlBOMResponseHandler_old = function(data) {
	switch(this.phase) {
		case this.ZERO_CONTACT:
			this.phase = this.FIRST_CONTACT;
			datafox_constants.redirect.lastIndex = 0;
			if (data.match(datafox_constants.redirect) == null) {
				//maybe no redirect, try with the default domain
				this.domain = "register.mtnl.net.in";
			} else {
				this.domain = datafox_constants.redirect.exec(data)[1];
			}
			this.domain += "/mtnlWebInt";
			//falls through
		case this.FIRST_CONTACT:
			this.phase = this.LOGIN;
			var pData = "uname=" + this.username + "&passwd=" + this.password;
			this.httpUtils.talkToServer("http://" + this.domain + "/userLogin", pData, null, null, this);
			break;
		case this.LOGIN:
			if (data.indexOf("Error Password incorrect...") != -1) {
				this.setState(this.ERROR, "Check your username or password");
				return;
			} else if (data.indexOf("domenudo") == -1) {
				//some other error
				this.setState(this.ERROR, "Usage data N/A");
				return;
			}
			this.phase = this.GET_SUPPLIES;
			//talkToServer("http://" + this.domain + "/domenudo", "submitBtn=View+Session+History", null, null, this); //advanced
			this.httpUtils.talkToServer("http://" + this.domain + "/domenudo", "submitBtn=View+Usage+Summary", null, null, this);
			break;
		case this.GET_SUPPLIES:
			this.parseMTNLBOMUsageData(data);
			break;
	}
};

DFox.prototype.mtnlBOMResponseHandler_new = function(data) {
	switch(this.phase) {
		case this.FIRST_CONTACT:
			this.phase = this.LOGIN;
			var pData = "loginMode=2&username=" + this.username + "&password=" + this.password + "&myAction=&Submit2=Login";
			this.httpUtils.talkToServer("http://" + this.domain + "/servlet/MyAccountLogin", pData, null, null, this);
			break;
		case this.LOGIN:
			if (data.indexOf("password is incorrect") != -1) {
				this.setState(this.ERROR, "Incorrect username or password");
				return;
			}
			var temp = this.execRE(datafox_constants.bstrAccId, data);
			if (!temp) {
				this.setState(this.ERROR, "Usage data N/A");
				return;
			}
			var accId = temp[1];
			var now = new Date();
			var monthYear = "+" + datafox_constants.month[now.getMonth()] + "%2C+" + now.getFullYear();
			var period = "&c_dateCreateFrom=" + "01" + monthYear + "&c_dateCreateTo=" + this.daysInMonth(now.getYear(), now.getMonth()) + monthYear;
			this.mtnlBOMRecordURL_new = "http://" + this.domain + "/servlet/Accounting?c_strActionMode=3217563&h_strAccountId=" + accId + "&c_strAccountId=" + accId;
			this.mtnlBOMActiveURL_new = "http://" + this.domain + "/servlet/Accounting?c_strActionMode=3217564&c_strAccountId=" + accId + "&c_strServiceId=SRV00003&c_strServiceName=Data&c_strDateTimeFormat=dd+MMM%2C+yyyy&c_btnSumbit=Search" + period;
			this.httpUtils.talkToServer(this.mtnlBOMActiveURL_new, null, null, null, this);
			this.phase = this.GET_SUPPLIES;
			break;
		case this.GET_SUPPLIES:
			try {
				this.parseMTNLBOMUsageData_new(data);
			} catch (e) {
				this.setState(this.ERROR, "Usage data N/A");
				this.log(e);
			}
			break;
	}
};

DFox.prototype.parseMTNLBOMUsageData_new = function(aData) {
	var gtidx = aData.indexOf("Grand Total");
	if (gtidx != -1) {
		var start = aData.lastIndexOf("<tr>", gtidx);
		var end = aData.indexOf("</tr>", gtidx);
		aData = aData.substring(start, end+5);
		aData = aData.replace(/&nbsp;/g, "");
		var parser = new DOMParser();
		var dom = parser.parseFromString(aData, "text/xml");
		if (dom.firstChild.nodeName == "parsererror") {
			//try to fix it, there is a known missing </td>
			aData = aData.replace("</tr>", "</td></tr>");
			dom = parser.parseFromString(aData, "text/xml");
			if (dom.firstChild.nodeName == "parsererror") {
				throw "Parser error: " + dom.firstChild.textContent;
			}
		}

		var cProps = new Object();
		var time = new Date();

		var temp = dom.evaluate("//td[3]", dom, null, XPathResult.STRING_TYPE, null).stringValue;
		cProps["dtS"] = parseFloat(temp) * 1024;
		temp = dom.evaluate("//td[4]", dom, null, XPathResult.STRING_TYPE, null).stringValue;
		cProps["dtR"] = parseFloat(temp) * 1024;
		temp = dom.evaluate("//td[5]", dom, null, XPathResult.STRING_TYPE, null).stringValue;
		cProps["dtX"] = parseFloat(temp) * 1024;
		temp = dom.evaluate("//td[6]", dom, null, XPathResult.STRING_TYPE, null).stringValue;
		cProps["dcX"] = cProps["dnX"] = parseFloat(temp) * 1024;
		cProps["stT"] = this.formatTime(time);
		cProps["ntC"] = time.getTime();
		this.setState(this.LOGGEDIN);

		var tProps = this.completeProgressCalculations(cProps["dnX"], cProps["ntC"]);
		var prop;
		for (prop in tProps) {
			cProps[prop] = tProps[prop];
		}
		var eqString = this.createEquateString("this.cProps", cProps);
		this.cacheDetails(eqString);
		this.push(eqString + "this.cached=false;");
		this.refreshUI();
	} else {
		throw "Could not find summary";
	}
};

DFox.prototype.parseMTNLBOMUsageData = function(aData) {
	datafox_constants.bMBUsed.lastIndex = 0;
	datafox_constants.adslFree.lasIndex = 0;
	datafox_constants.nightMB.lastIndex = 0;

	var used, night, free;

	try {
		used  = parseFloat(datafox_constants.bMBUsed.exec(aData)[1]) * 1024;
		//free = parseFloat(adslFree.exec(aData)[1]) * 1024;
	} catch (e) {
		this.setState(this.ERROR, "Usage data N/A");
		return;
	}

	var temp = new Date();
	var cProps = new Object();
	cProps["dtR"] = used;
	cProps["dcX"] = cProps["dnX"] = used;
	cProps["drR"] = free;
	cProps["stT"] = this.formatTime(temp);
	cProps["ntC"] = temp.getTime();

	this.setState(this.LOGGEDIN);
	var tProps = this.completeProgressCalculations(used, cProps["ntC"], this.dateString);
	var prop;
	for (prop in tProps) {
		cProps[prop] = tProps[prop];
	}
	var eqString = this.createEquateString("this.cProps", cProps);
	this.cacheDetails(eqString);
	this.push(eqString + "this.cached=false;");
	this.refreshUI();
};

DFox.prototype.mtnlDELLogin = function() {
	this.mtnlDate = new Date();

	this.phase = this.LOGIN;
	var pData = "login=" + this.username + "&password=" + this.password + "&service=mtnl%2Fadsl";
	this.domain = "register.bol.net.in/htmlui_new";
	this.httpUtils.talkToServer("http://" + this.domain + "/login_verify.jsp", pData, null, null, this, false, 5*1000);
};


DFox.prototype.mtnlDELResponseHandler = function(data) {
	switch(this.phase) {
		case this.LOGIN:
			this.phase = this.GET_SUPPLIES;
			if (data.length == 0) {
				this.log("Error logging in");
				this.setState(this.ERROR, "MTNL site not responding");
				break;
			}
			if (data.indexOf("Successful Login") == -1) {
				this.log("Error logging in");
				this.setState(this.ERROR, "Check your username and password");
				break;
			}
			this.httpUtils.talkToServer("http://" + this.domain + "/PostpaidViewAcctInfo.jsp", null, null, null, this);
			break;
		case this.GET_SUPPLIES:
			/*var pData = "startday=1&startmonth=2&startyear=2007&endday=28&endmonth=2&endyear=2007"; //&referrer=SessionHistoryServlet";
			this.push(this.createEquateString("this.openURLPD", pData));
			talkToServer("http://" + this.domain + "/SessionHistoryServlet", pData, null, null, this);*/
			this.parseMTNLDELUsageData(data);
			break;
	}
}

DFox.prototype.parseMTNLDELUsageData = function(aData) {
	datafox_constants.mbUsed.lastIndex = 0;
	datafox_constants.adslFree.lastIndex = 0;
	datafox_constants.nightMB.lastIndex = 0;

	var used, night, free;

	try {
		used  = parseFloat(datafox_constants.mbUsed.exec(aData)[1]) * 1024;
		free = parseFloat(datafox_constants.adslFree.exec(aData)[1]) * 1024;
		var temp = datafox_constants.nightMB.exec(aData)
		if (temp && temp.length == 2) {
			night = parseFloat(temp[1]) * 1024;
		}
	} catch (e) {
		this.setState(this.ERROR, "Usage data N/A");
		return;
	}

	var temp = new Date();
	var cProps = new Object();
	cProps["dtR"] = used + (!isNaN(night) ? night : 0);
	cProps["dcX"] = cProps["dnX"] = used;
	cProps["drR"] = free;
	cProps["stT"] = this.formatTime(temp);
	cProps["ntC"] = temp.getTime();

	this.setState(this.LOGGEDIN);
	var tProps = this.completeProgressCalculations(used, cProps["ntC"], this.dateString);
	var prop;
	for (prop in tProps) {
		cProps[prop] = tProps[prop];
	}
	var eqString = this.createEquateString("this.cProps", cProps);
	this.cacheDetails(eqString);
	this.push(eqString + "this.cached=false;");
	this.refreshUI();
};


DFox.prototype.mtnlBOMOpenInBrowser = function(openRecords) {
	if (!this.newBOMPortal) {
		var pd = openRecords 	? "submitBtn=Last+Month+Sessions"
								: "submitBtn=View+Session+History";
		this.httpUtils.openInBrowser("http://register.mtnl.net.in/mtnlWebInt/domenudo", pd);
	} else {
		this.httpUtils.openInBrowser(openRecords ? this.mtnlBOMRecordURL_new : this.mtnlBOMActiveURL_new);
	}
};

DFox.prototype.mtnlDELOpenInBrowser = function(openRecords) {
	var sDate = new Date();
	var eDate = new Date();
	eDate.setTime(sDate.getTime());
	sDate.setDate(1);
	if (openRecords) {
		//open last month's details
		sDate.setMonth(sDate.getMonth()-1);
		eDate.setDate(0);
	}

	var pData = "startday=" + sDate.getDate() + "&startmonth=" + (sDate.getMonth() + 1) + "&startyear=" + sDate.getFullYear() +
				"&endday=" + eDate.getDate() + "&endmonth=" + (eDate.getMonth() + 1) + "&endyear=" + eDate.getFullYear();
	this.log(pData);
	this.httpUtils.openInBrowser("http://" + this.domain + "/SessionHistoryServlet", pData, null, null, this);
};