/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2008 Brian Fernandes <infernalproteus@gmail.com>

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/

var datafox_PMgr = {
	pMgrI: null,
	pMgr: null,
	FORM_URL: "User management",
	
	init: function() {
		var temp = Components.classes["@mozilla.org/passwordmanager;1"];
		if (temp != null) {
			//Firefox 2
			this.pMgrI = temp.getService(Components.interfaces.nsIPasswordManagerInternal);
			this.pMgr = temp.getService(Components.interfaces.nsIPasswordManager);
		} else {
			//Firefox 3
			this.lMgr = Components.classes["@mozilla.org/login-manager;1"].getService(Components.interfaces.nsILoginManager);
			this.nsLoginInfo = new Components.Constructor("@mozilla.org/login-manager/loginInfo;1",
                                             Components.interfaces.nsILoginInfo,
                                             "init");	  
		}
	},
	
	getSavedPassword: function(username) {
		var pWord;
		if (this.pMgrI != null) {
			var unwanted = {value:""};
			var password = {value:""};
			try {
				this.pMgrI.findPasswordEntry(dataFox.name, username, null, unwanted, unwanted, password);
			} catch(e) {
				//do nothing
			}
			if (password.value && password.value.length > 0) return password.value;
			return null;			
		} else {
			var logins = this.lMgr.findLogins({}, dataFox.name, this.FORM_URL, null);
			for (i = 0; i < logins.length; i++) {
				if (logins[i].username == username) {
					var password = logins[i].password;
					return (password && password.length > 0) ? password : null;
				}
			}
			return null;
		}		
	},
	
	getUserNames: function() {
		var users = new Array();
		if (this.pMgr != null) {
			var enumerator = this.pMgr.enumerator;
			while(enumerator.hasMoreElements()) {
				var np;
				try {
					np = enumerator.getNext();
				} catch(e) {
					break;
				}
				
				np = np.QueryInterface(Components.interfaces.nsIPassword);
				if (np.host == dataFox.name) {
					try {
						var user = np.user;
						users.push(user);
					} catch (e) {				
					}
				}
			}
		} else {
			var logins = this.lMgr.findLogins({}, dataFox.name, this.FORM_URL, null);
			for (i = 0; i < logins.length; i++) {
				users.push(logins[i].username);
			}
		}
		return users;
	},
	
	managePassword: function (username, password, savePassword) {
		if (this.pMgr != null) {
			try {
				this.pMgr.removeUser(dataFox.name, username);
			} catch (e) {
				//do nothing
			}				
			this.pMgr.addUser(dataFox.name, username, savePassword ? password : "");
		} else {
			var newLoginInfo = new this.nsLoginInfo(dataFox.name, this.FORM_URL, null, username, savePassword ? password : "", "", "");
			var logins = this.lMgr.findLogins({}, dataFox.name, this.FORM_URL, null);
			for (i = 0; i < logins.length; i++) {
				if (logins[i].username == username) {
					//could try using modifyLogin here, but doesn't seem worth it
					this.lMgr.removeLogin(logins[i]);
					break;
				}
			}
			if (savePassword) {
				//TODO: Unlike FF2, we can't save empty passwords in FF3, so using the password manager just to store the list of users does not work anymore
				try {
					this.lMgr.addLogin(newLoginInfo);
				} catch (e) {
					//do nothing
				}
			}
		}
	}
};