/*
 	This file is part of DataFox, a bandwidth usage monitor for 
 	BSNL DataOne & MTNL Triband.
 	Copyright (C) 2007-2008 Brian Fernandes <infernalproteus@gmail.com>

 	This file contains adapted versions of functions from the Gmail Notifer 
 	extension by Doron Rosenberg. 

    DataFox is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DataFox is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DataFox.  If not, see <http://www.gnu.org/licenses/>
*/
function datafox_prefs() {
	this.prefBranch = null;
	this.DEBUG = "datafox.debug.active";
	this.ACTIVE_USER = "datafox.active.user";
	this.ACTIVE_DOMAIN = "datafox.active.domain";
	this.ACTIVE_ISP = "datafox.active.isp";
	this.OVERRIDE_DOMAIN = "datafox.override.domain";
	this.SAVE_PASSWORD = "datafox.save.password";
	this.TOOLTIP_STRING = "datafox.tooltip.string";
	this.STATUS_STRING = "datafox.status.string";
	this.DATA_UNIT = "datafox.data.unit";
	this.LOGIN_STARTUP = "datafox.login.startup";
	this.CACHE_DATA = "datafox.cache.data";
	this.DECIMAL_PLACES = "datafox.decimal.places";
	this.BANDWIDTH_LIMIT = "datafox.bandwidth.limit";
	this.TEXT_COOL_COLOR = "datafox.text.cool";
	this.TEXT_UNCOOL_COLOR = "datafox.text.uncool";
	this.TEXT_CACHED_COLOR = "datafox.text.cached";
	this.PROG_COOL_COLOR = "datafox.prog.cool";
	this.PROG_UNCOOL_COLOR = "datafox.prog.uncool";
	this.BKGND_COOL_COLOR = "datafox.bkgnd.cool";
	this.BKGND_UNCOOL_COLOR = "datafox.bkgnd.uncool";
	this.PROG_ENABLE = "datafox.prog.enable";
	this.BKGND_ENABLE = "datafox.bkgnd.enable";
	this.USAGE_GRANULARITY = "datafox.usage.granularity";
}

datafox_prefs.prototype.getPrefBranch = function() {
	if (!this.prefBranch) {
		this.prefBranch = Components.classes['@mozilla.org/preferences-service;1'].getService();
		this.prefBranch = this.prefBranch.QueryInterface(Components.interfaces.nsIPrefBranch);
  	}
  	return this.prefBranch;
};

datafox_prefs.prototype.setCharPref = function(name, value) {
	this.getPrefBranch().setCharPref(name, value);
};

datafox_prefs.prototype.getCharPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getCharPref(name);
		if (value && value.length == 0) value = null;		
	} catch (e) {
	}
	return value;	
};

datafox_prefs.prototype.setBoolPref = function(name, value) {
	this.getPrefBranch().setBoolPref(name, value);
};

datafox_prefs.prototype.getBoolPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getBoolPref(name);
	} catch (e) {
	}
	return value;	
};

datafox_prefs.prototype.setIntPref = function(name, value) {
	this.getPrefBranch().setIntPref(name, value);
};

datafox_prefs.prototype.getIntPref = function(name) {
	var value = null;
	try {
		value = this.getPrefBranch().getIntPref(name);
	} catch (e) {
	}
	return value;	
};


datafox_prefs.prototype.initPrefs = function() {
	this.initPref(this.ACTIVE_DOMAIN, "char", "10.240.144.195");	
	this.initPref(this.TOOLTIP_STRING, "char", "Up: {tS}GB Down: {tR}GB\nChargeable transfer: {cX}GB ({uP}%)\nSurplus transfer: {sX}GB\nRemaining transfer: {rX}GB\nHours: {tH} Period: {YM}\nLast updated: {tT}");
	this.initPref(this.STATUS_STRING, "char", "{cX}GB");
	this.initPref(this.DEBUG, "bool", false);
	this.initPref(this.ACTIVE_ISP, "int", 0);
	this.initPref(this.DATA_UNIT, "int", 0);
	this.initPref(this.DECIMAL_PLACES, "int", 3);
	this.initPref(this.LOGIN_STARTUP, "bool", true);
	this.initPref(this.BANDWIDTH_LIMIT, "int", 0);
	this.initPref(this.TEXT_COOL_COLOR, "char", "#000000");
	this.initPref(this.TEXT_UNCOOL_COLOR, "char", "#FF0000");
	this.initPref(this.TEXT_CACHED_COLOR, "char", "#808080");
	this.initPref(this.PROG_COOL_COLOR, "char", "#CCFFFF");
	this.initPref(this.PROG_UNCOOL_COLOR, "char", "#FFCC99");
	this.initPref(this.BKGND_COOL_COLOR, "char", "#CCCCCC");
	this.initPref(this.BKGND_UNCOOL_COLOR, "char", "#666666");
	this.initPref(this.PROG_ENABLE, "bool", true);
	this.initPref(this.BKGND_ENABLE, "bool", false);
	this.initPref(this.USAGE_GRANULARITY, "int", 0);	
};

datafox_prefs.prototype.initPref = function(name, type, dValue) {
	switch(type) {
		case "char":
			var ePref = this.getCharPref(name);
			if (!ePref || ePref.length == 0) {
				this.setCharPref(name, dValue);
			}
			break;
		case "bool":
      		var ePref = this.getBoolPref(name);
      		if (ePref == null) {
        		this.setBoolPref(name, dValue);
        	}
      		break;
      	case "int":
      		var ePref = this.getIntPref(name);
      		if (ePref == null) {
        		this.setIntPref(name, dValue);
        	}
      		break;      	
	}
};

datafox_prefs.prototype.dfPrefObserver = {
  register: function() {
    this._branch = dataFox.dfPrefs.getPrefBranch();
//    this._branch.QueryInterface(Components.interfaces.nsIPrefBranch2);
    this._branch.addObserver("", this, false);
  },

  unregister: function() {
    if(!this._branch) return;
    this._branch.removeObserver("", this);
  },

  observe: function(aSubject, aTopic, aData) {
    if(aTopic != "nsPref:changed") return;
    // aSubject is the nsIPrefBranch we're observing (after appropriate QI)
    // aData is the name of the pref that's been changed (relative to aSubject)
    switch (aData) {
      case dataFox.dfPrefs.DECIMAL_PLACES:
      	var places = dataFox.dfPrefs.getIntPref(dataFox.dfPrefs.DECIMAL_PLACES);
      	datafox_constants.ROUND_TO = Math.pow(10, places);
      	//falls through      	
      case dataFox.dfPrefs.TOOLTIP_STRING:
      case dataFox.dfPrefs.STATUS_STRING:
      case dataFox.dfPrefs.DATA_UNIT:
      	//do not call refreshUI because each window recevies this notification anyway
      	dataFox.updateStrings();      
        break;
      case dataFox.dfPrefs.TEXT_COOL_COLOR:
      case dataFox.dfPrefs.TEXT_UNCOOL_COLOR:
      case dataFox.dfPrefs.TEXT_CACHED_COLOR:
      	dataFox.updateTextColor();
      	break;
      case dataFox.dfPrefs.BKGND_COOL_COLOR:
      case dataFox.dfPrefs.BKGND_UNCOOL_COLOR:
      case dataFox.dfPrefs.PROG_COOL_COLOR:
      case dataFox.dfPrefs.PROG_UNCOOL_COLOR:
      	dataFox.updateProgressMeterUI();
      	break;
      case dataFox.dfPrefs.BANDWIDTH_LIMIT:
      case dataFox.dfPrefs.PROG_ENABLE:
      case dataFox.dfPrefs.BKGND_ENABLE:
      case dataFox.dfPrefs.USAGE_GRANULARITY:
      	dataFox.updateProgressMeter();
      	break;
    }
  }
};
